import com.comsol.model.*;
import com.comsol.model.util.*;

/** Model exported on Aug 12 2020, 14:57 by COMSOL 5.4.0.388. */
public class Lag_sme_mart{
      public static Model run() {
          Model model = ModelUtil.create("Model");

          model.modelPath("/home/apadhy/projects/PhD/COMSOL");
          model.label("Lag_sme_mart.mph");
// Global Definitions block

          model.param().set("t1", "0.01 [s]"); // Ramping starts
          model.param().set("t2", "0.25 [s]"); // Saturation
          model.param().set("t3", "0.5 [s]"); // Reduction
          model.param().set("t4", "0.75 [s]"); // Temperature increase and load set to zero
          model.param().set("t5", "2.0 [s]");

          model.param().set("dt", "0.001 [s]");

          model.param().set("T_ref", "280[K]", "Applied Temperature");
          model.param().set("T_rec", "320[K]", "Recovery Temperature");

          model.param().set("width", "15[mm]", "Length");
          model.param().set("height", "1.5[mm]", "width");

          model.param().set("d", "0.030[mm]", "thickness");
          model.param().set("maxload", "4e8 [Pa]", "Max pressure on edge");

          model.param().set("p_ratio", "0.33", "Poissonsratio");
          model.param().set("rho_", "6500", "Density");
          model.param().set("Ymod_A", "41.2e9", "Youngs Modulus Austenite");
          model.param().set("Ymod_M", "32.9e9", "Youngs Modulus Martensite");
          model.param().set("Hcap", "400", "Heat capacity");
          model.param().set("kappa_A", "18.0", "Thermal conductivity Austenite");
          model.param().set("kappa_M", "8.6", "Thermal conductivity Martensite");
          model.param().set("hflux", "18.0", "Heat Flux coefficient");

          model.param().set("TMs_1", "291", "Martensite start temperature");
          model.param().set("TMf_1", "271", "Martensite finish temperature");
          model.param().set("TAs_1", "295", "Austenite start temperature");
          model.param().set("TAf_1", "315", "Austenite finish temperature");

          model.param().set("C_M", "4e6", "Martensite slope");
          model.param().set("C_A", "4e6", "Austenite slope");
          model.param().set("etr_max", "0.04", "max transformation strain");
          model.param().set("xi_0", "1.0", "Initial martensite fraction");


// Displacement Function Piecewise

          model.func().create("pw1", "Piecewise");
          model.func("pw1").label("pw1");
          model.func("pw1").set("arg", "t");
          model.func("pw1").set("smooth", "contd2");
          model.func("pw1").set("smoothzone", "0.001");
          model.func("pw1")
          .set("pieces", new String[][]{
              {"0", "t1", "1e-8"},
              {"t1", "t2", "maxload/(t2-t1)*(t-t1)"},
              {"t2", "t3", "maxload"},
              {"t3", "t4", "maxload/(t3-t4)*(t-t4)"},
              {"t4", "t5", "1e-8"}});

// Function temperature interpolation
          model.func().create("pw2", "Piecewise");
          model.func("pw2").label("pw2");
          model.func("pw2").set("arg", "t");
          model.func("pw2").set("smooth", "contd2");
          model.func("pw2").set("smoothzone", "0.001");
          model.func("pw2")
          .set("pieces", new String[][]{
              {"0", "t1", "T_ref"},
              {"t1", "t2", "T_ref"},
              {"t2", "t3", "T_ref"},
              {"t3", "t4", "T_ref"},
              {"t4", "t5", "((T_rec-T_ref)/(t5-t4))*(t-t4)+T_ref"}});

// Function for plotting Lagoudas Phase modelPath
          model.func().create("Ms", "Analytic");
          model.func("Ms").label("Martensite Start boundary");
          model.func("Ms").set("funcname", "Ms");
          model.func("Ms").set("expr", "TMs_1+C_M*(T-TMs_1)");
          model.func("Ms").set("args", "T");
        
          model.func("Ms").setIndex("plotargs", "TMf_1", 0, 1);
          model.func("Ms").setIndex("plotargs", "TAf_1", 0, 2);

          ////
          model.func().create("Mf", "Analytic");
          model.func("Mf").label("Martensite finish boundary");
          model.func("Mf").set("funcname", "Mf");
          model.func("Mf").set("expr", "TMf_1+C_M*(T-TMf_1)");
          model.func("Mf").set("args", "T");
        
          model.func("Mf").setIndex("plotargs", "TMf_1", 0, 1);
          model.func("Mf").setIndex("plotargs", "TAf_1", 0, 2);
          
          
          ////
          model.func().create("As", "Analytic");
          model.func("As").label("Austenite Start boundary");
          model.func("As").set("funcname", "As");
          model.func("As").set("expr", "TAs_1+C_A*(T-TAs_1)");
          model.func("As").set("args", "T");
        
          model.func("As").setIndex("plotargs", "TMf_1", 0, 1);
          model.func("As").setIndex("plotargs", "TAf_1", 0, 2);
          
          
          model.func().create("Af", "Analytic");
          model.func("Af").label("Austenite finish boundary");
          model.func("Af").set("funcname", "Af");
          model.func("Af").set("expr", "TAf_1+C_A*(T-TAf_1)");
          model.func("Af").set("args", "T");
        
          model.func("Af").setIndex("plotargs", "TMf_1", 0, 1);
          model.func("Af").setIndex("plotargs", "TAf_1", 0, 2);
          
          
// Function Random normal distribution

          model.func().create("rn1", "Random");
          model.func("rn1").set("nargs", 2);
          model.func("rn1").set("type", "normal");
          model.func("rn1").set("funcname", "rn");


// Component block
          model.component().create("comp1", true);
          model.component("comp1").geom().create("geom1", 2);

//Geometry block
          model.component("comp1").geom("geom1").lengthUnit("mm");
          model.component("comp1").geom("geom1").create("r1", "Rectangle");
          model.component("comp1").geom("geom1").feature("r1").set("size", new String[]{"width", "height"});
          model.component("comp1").geom("geom1").feature("fin").label("Form Union1");
          model.component("comp1").geom("geom1").feature("fin").set("action", "union");
          model.component("comp1").geom("geom1").run();
          // model.component("comp1").geom("geom1").run("fin");


// Definitions
          model.component("comp1").cpl().create("aveop1", "Average");
          model.component("comp1").cpl("aveop1").selection().set(1);

          model.component("comp1").coordSystem("sys1").setIndex("coord", "t1", 0, 0);
          model.component("comp1").coordSystem("sys1").setIndex("coord", "n", 0, 1);
          model.component("comp1").coordSystem("sys1").setIndex("coord", "to", 0, 2);

          model.component("comp1").view("view1");
          model.component("comp1").view("view1").set("showgrid", true);
          model.component("comp1").view("view1").set("rendermesh", true);
          model.component("comp1").view("view1").set("showunits", true);
          model.component("comp1").view("view1").set("showselection", true);

		  model.component("comp1").cpl().create("aveop2", "Average");
		  model.component("comp1").cpl("aveop2").selection().geom("geom1", 1);
		  model.component("comp1").cpl("aveop2").selection().set(4);
		  model.component("comp1").cpl("aveop2").label("AverageBoundary");



 //Material block Austenite
          model.component("comp1").material().create("mat1", "Common");
          model.component("comp1").material("mat1").label("Austenite");
          model.component("comp1").material("mat1").propertyGroup("def").set("poissonsratio", "p_ratio");
          model.component("comp1").material("mat1").propertyGroup("def").set("density", "rho_");
          model.component("comp1").material("mat1").propertyGroup("def").set("youngsmodulus", "Ymod_A");
          model.component("comp1").material("mat1").propertyGroup("def").set("heatcapacity", "Hcap");
          model.component("comp1").material("mat1").propertyGroup("def").set("thermalconductivity", "kappa_A");

//Material block Martensite
          model.component("comp1").material().create("mat2", "Common");
          model.component("comp1").material("mat2").label("Martensite");
          model.component("comp1").material("mat2").propertyGroup("def").set("poissonsratio", "p_ratio");
          model.component("comp1").material("mat2").propertyGroup("def").set("density", "rho_");
          model.component("comp1").material("mat2").propertyGroup("def").set("youngsmodulus", "Ymod_M");
          model.component("comp1").material("mat2").propertyGroup("def").set("heatcapacity", "Hcap");
          model.component("comp1").material("mat2").propertyGroup("def").set("thermalconductivity","kappa_M");
          model.component("comp1").material("mat2").selection().all();

// Heat transfer
          model.component("comp1").physics().create("ht", "HeatTransfer", "geom1");
          model.component("comp1").physics("ht").selection().set(1);

          model.component("comp1").physics("ht").prop("EquationForm").setIndex("form", "Automatic", 0);

          model.component("comp1").physics("ht").prop("PhysicalModelProperty").set("dz", "d");
          model.component("comp1").physics("ht").prop("PhysicalModelProperty").set("Tref", "T_ref");
          model.component("comp1").physics("ht").prop("PhysicalModelProperty").set("HeatTransferInAlloys", true);

          // sma1
          model.component("comp1").physics("ht").create("sma1", "ShapeMemoryAlloy", 2);
          model.component("comp1").physics("ht").feature("sma1").selection().set(1);

          model.component("comp1").physics("ht").feature("sma1").set("EffectiveConductivity", "PowerLaw");
          model.component("comp1").physics("ht").feature("sma1").set("MartensiteMaterial", "mat2");
          model.component("comp1").physics("ht").feature("sma1").set("AusteniteMaterial", "mat1");
          model.component("comp1").physics("ht").feature("sma1").set("xi", "xi_0");
          model.component("comp1").physics("ht").feature("sma1").set("rho_mat", "userdef");
          model.component("comp1").physics("ht").feature("sma1").set("rho", "rho_");


          // temp1
          model.component("comp1").physics("ht").create("temp1", "TemperatureBoundary", 1);
          model.component("comp1").physics("ht").feature("temp1").selection().set(1, 4);
          model.component("comp1").physics("ht").feature("temp1").set("T0", "T_ref");

//          //init1
          model.component("comp1").physics("ht").feature("init1").set("Tinit", "T_ref");

          // hf1
          model.component("comp1").physics("ht").create("hf1", "HeatFluxBoundary", 1);

          model.component("comp1").physics("ht").feature("hf1").selection().set(2, 3);
          model.component("comp1").physics("ht").feature("hf1").set("materialType", "solid");
          model.component("comp1").physics("ht").feature("hf1").set("HeatFluxType", "ConvectiveHeatFlux");
          model.component("comp1").physics("ht").feature("hf1").set("h", "hflux");
          model.component("comp1").physics("ht").feature("hf1").set("Text", "T_ref");

          //ophf1
          model.component("comp1").physics("ht").create("ophf1", "OutOfPlaneHeatFlux", 2);
          model.component("comp1").physics("ht").feature("ophf1").selection().set(1);

          model.component("comp1").physics("ht").feature("ophf1").set("UpsideHeatFluxType", "ConvectiveHeatFlux");
          model.component("comp1").physics("ht").feature("ophf1").set("h_u", "hflux");
          model.component("comp1").physics("ht").feature("ophf1").set("Text_u", "T_ref");

          model.component("comp1").physics("ht").feature("ophf1").set("DownsideHeatFluxType", "ConvectiveHeatFlux");
          model.component("comp1").physics("ht").feature("ophf1").set("h_d", "hflux");
          model.component("comp1").physics("ht").feature("ophf1").set("Text_d", "T_ref");

          // hs1
          model.component("comp1").physics("ht").create("hs1", "HeatSource", 2);
          model.component("comp1").physics("ht").feature("hs1").selection().set(1);

          model.component("comp1").physics("ht").feature("hs1").set("materialType", "from_mat");
          model.component("comp1").physics("ht").feature("hs1").set("heatSourceType", "GeneralSource");

// Solid creation
          model.component("comp1").physics().create("solid", "SolidMechanics", "geom1");
          model.component("comp1").physics("solid").selection().set(1);

          model.component("comp1").physics("solid").prop("EquationForm").setIndex("form", "Automatic", 0);

          model.component("comp1").physics("solid").prop("Type2D").set("Type2D", "PlaneStress");
          model.component("comp1").physics("solid").prop("d").set("d", "d");
          model.component("comp1").physics("solid").prop("ShapeProperty").set("order_displacement", 2);
          model.component("comp1").physics("solid").prop("StructuralTransientBehavior")
          .set("StructuralTransientBehavior", "IncludeInertia");
          model.component("comp1").physics("solid").field("displacement").field("UD");


          /* Displacement order 1
           model.component("comp1").physics("solid").prop("ShapeProperty").set("order_displacement", 1);
          *
           * Static Simulation
          model.component("comp1").physics("solid").prop("StructuralTransientBehavior")
         .set("StructuralTransientBehavior", "Quasistatic");
           *
           *Reference point for Moment Computation
           *
  		model.component("comp1").physics("solid").prop("refpnt").set("refpnt", new double[]{0.1, 0.1, 0});
  	          * Auricchio Model
  	          model.component("comp1").physics("solid").feature("sma1").set("ShapeMemoryAlloyModel", "SouzaAuricchio");
           * Cosine
             model.component("comp1").physics("solid").feature("sma1").set("TransformationFunction", "Cosine");
           *Initial martensite fraction
               model.component("comp1").physics("solid").feature("sma1").set("xi0", 0.1);
               *Changing variables in equation view
              model.component("comp1").physics("solid").feature("sma1").featureInfo("info")
              .set("solid.eYZ", 0, new String[]{"0.1"});
           */


          // sma1
          model.component("comp1").physics("solid").create("sma1", "ShapeMemoryAlloy", 2);
          model.component("comp1").physics("solid").feature("sma1").set("minput_temperature_src", "userdef");
          model.component("comp1").physics("solid").feature("sma1").set("minput_temperature", "pw2(t)");
          model.component("comp1").physics("solid").feature("sma1").selection().set(1);

          model.component("comp1").physics("solid").feature("sma1").set("ShapeMemoryAlloyModel", "Lagoudas");
          model.component("comp1").physics("solid").feature("sma1").set("T0", "T_ref");
          model.component("comp1").physics("solid").feature("sma1").set("AusteniteMaterial", "mat1");
          model.component("comp1").physics("solid").feature("sma1").set("MartensiteMaterial", "mat2");

          model.component("comp1").physics("solid").feature("sma1").set("TMs", "TMs_1+rn(x,y)");
          model.component("comp1").physics("solid").feature("sma1").set("TMf", "TMf_1+rn(x,y)");
          model.component("comp1").physics("solid").feature("sma1").set("CM", "C_M");

          model.component("comp1").physics("solid").feature("sma1").set("TAs", "TAs_1+rn(x,y)");
          model.component("comp1").physics("solid").feature("sma1").set("TAf", "TAf_1+rn(x,y)");
          model.component("comp1").physics("solid").feature("sma1").set("CA", "C_A");

          model.component("comp1").physics("solid").feature("sma1").set("etrmax", "etr_max");
          model.component("comp1").physics("solid").feature("sma1").set("xi0", "xi_0");

          model.component("comp1").physics("solid").feature("sma1").set("TransformationFunction", "Quadratic");
//          model.component("comp1").physics("solid").feature("sma1").feature("trdir1")
//          .set("TransformationDirectionType", "Automatic");
          model.component("comp1").physics("solid").feature("sma1").set("TransformationDirectionType", "UserDefined");
          model.component("comp1").physics("solid").feature("sma1").set("trdir", "if(t<t3,1,-1)");

          model.component("comp1").physics("solid").feature("sma1").set("CalculateDissipatedEnergy", true);

          // Initial values


          // fix1
//          model.component("comp1").physics("solid").create("fix1", "Fixed", 1);
//          model.component("comp1").physics("solid").feature("fix1").selection().set(1);
//          model.component("comp1").physics("solid").feature("fix1").active(false);

          // disp1
          model.component("comp1").physics("solid").create("disp1", "Displacement1", 1);
          model.component("comp1").physics("solid").feature("disp1").selection().set(1);

          model.component("comp1").physics("solid").feature("disp1").setIndex("Direction", true, 0);
          model.component("comp1").physics("solid").feature("disp1").setIndex("Direction", true, 1);


//          // disp2
//          model.component("comp1").physics("solid").create("disp2", "Displacement1", 1);
//          model.component("comp1").physics("solid").feature("disp2").selection().set(4);
//
//          model.component("comp1").physics("solid").feature("disp2").setIndex("Direction", true, 0);
//          model.component("comp1").physics("solid").feature("disp2").setIndex("Direction", true, 1);
//          model.component("comp1").physics("solid").feature("disp2").setIndex("U0", "pw1(t)", 0);

          // Boundary load
          model.component("comp1").physics("solid").create("bndl1", "BoundaryLoad", 1);
          model.component("comp1").physics("solid").feature("bndl1").selection().set(4);
          model.component("comp1").physics("solid").feature("bndl1").set("FperArea", new String[]{"pw1(t)", "0", "0"});



          //free2
          model.component("comp1").physics("solid").create("free2", "Free", 1);
          model.component("comp1").physics("solid").feature("free2").selection().set(2, 3);


          // Displacements activated
          model.component("comp1").physics("solid").feature("disp1").active(true);
//          model.component("comp1").physics("solid").feature("disp2").active(true);

          // Total power dissipation density Heat transfer in Solids
          model.component("comp1").physics("ht").feature("hs1").set("Q0_src", "root.comp1.solid.Qh");


//  Mesh
          model.component("comp1").mesh().create("mesh1");
          model.component("comp1").mesh("mesh1").create("map1", "Map");
          model.component("comp1").mesh("mesh1").feature("size").set("hauto", 1);
          model.component("comp1").mesh("mesh1").run();


// Study
          model.study().create("std1");
          model.study("std1").create("time", "Transient");

          model.study("std1").feature("time").set("tlist", "range(0,dt,t5)");
          model.study("std1").feature("time").set("usertol", true);
          model.study("std1").feature("time").set("plot", true);
          model.study("std1").feature("time").set("useadvanceddisable", true);
          model.study("std1").feature("time").activate("solid", true);


// Solution
          model.sol().create("sol1");
          model.sol("sol1").study("std1");
          model.sol("sol1").attach("std1");
          model.sol("sol1").create("st1", "StudyStep");
          model.sol("sol1").create("v1", "Variables");
          model.sol("sol1").create("t1", "Time");

          // Dependent variables v1
          model.sol("sol1").feature("v1").set("resscalemethod", "auto");
          model.sol("sol1").feature("v1").set("clist", new String[]{"range(0,dt,t5)", "0.002[s]"});

          // Time dependent solver t1
          model.sol("sol1").feature("t1").set("control", "user");
          model.sol("sol1").feature("t1").set("tlist", "range(0,dt,t5)");
          model.sol("sol1").feature("t1").set("atolglobalfactor", 0.001);
          model.sol("sol1").feature("t1").feature().remove("fcDef");

          model.sol("sol1").feature("t1").set("timemethod", "bdf");
          model.sol("sol1").feature("t1").set("tstepsbdf", "strict");

          model.sol("sol1").feature("t1").create("fc1", "FullyCoupled");
          model.sol("sol1").feature("t1").feature("fc1").set("dtech", "hnlin");
          model.sol("sol1").feature("t1").feature("fc1").set("maxiter", 20);
          model.sol("sol1").feature("t1").feature("fc1").set("dtech", "auto");

          // Martensite previous solution
//          model.component("comp1").physics().create("dode", "DomainODE", new String[][]{{"u2"}});
//
//          model.component("comp1").physics("dode").tag("Martensite");
//          model.component("comp1").physics("Martensite").prop("Units").set("CustomDependentVariableUnit", "1");
//          model.component("comp1").physics("Martensite").prop("Units").setIndex("CustomSourceTermUnit", 1, 0, 0);
//          model.component("comp1").physics("Martensite").field("dimensionless").component(1, "mar");
//          model.component("comp1").physics("Martensite").feature("dode1").setIndex("f", "mar-nojac(solid.xi_M)", 0);
//          model.component("comp1").physics("Martensite").feature("dode1").setIndex("da", 0, 0);
//          model.component("comp1").physics("Martensite").feature("init1").set("mar", "solid.xi0");
//
//          model.sol("sol1").feature("t1").create("ps1", "PreviousSolution");
//          model.sol("sol1").feature("t1").feature("ps1").set("prevcomp", new String[]{"comp1_mar"});
//
//          model.component("comp1").variable().create("var1");
//          model.component("comp1").variable("var1").set("m_dot", "aveop1(solid.xi_M)-aveop1(comp1.mar)");
//
//          model.component("comp1").physics("solid").feature("sma1").set("TransformationDirectionType", "UserDefined");
//          model.component("comp1").physics("solid").feature("sma1").set("trdir", "if(m_dot>0,1,-1)");

          model.sol("sol1").feature("t1").set("plot", true);
          //model.sol("sol1").runAll();

// Results
          model.result().create("pg1", "PlotGroup2D");

          model.result("pg1").create("surf1", "Surface");
          model.result("pg1").feature("surf1").label("Temperature");
          model.result("pg1").feature("surf1").set("data", "dset1");
          model.result("pg1").feature("surf1").active(false);
          
          model.result("pg1").create("surf2", "Surface");
          model.result("pg1").feature("surf2").label("Stress");
          model.result("pg1").feature("surf2").set("data", "dset1");
          model.result("pg1").feature("surf2").set("expr", "solid.mises");
          model.result("pg1").feature("surf2").active(false);

          model.result("pg1").create("surf3", "Surface");
          model.result("pg1").feature("surf3").label("Martensite fraction");
          model.result("pg1").feature("surf3").set("data", "dset1");
          model.result("pg1").feature("surf3").set("expr", "solid.xi_M");

          model.result("pg1").create("surf4", "Surface");
          model.result("pg1").feature("surf4").label("Austenite Fraction");
          model.result("pg1").feature("surf4").set("expr", "solid.xi_A");
          model.result("pg1").feature("surf4").active(false);
          model.result("pg1").run();


          model.result().create("pg2", "PlotGroup1D");
          model.result("pg2").label("Strain based");

          model.result("pg2").create("glob1", "Global");
          model.result("pg2").feature("glob1").setIndex("expr", "aveop1(solid.sx)", 0);
          model.result("pg2").feature("glob1").set("xdata", "expr");
          model.result("pg2").feature("glob1").set("xdataexpr", "aveop1(solid.eXX)");
          model.result("pg2").feature("glob1").label("Stress/strain");

          model.sol("sol1").feature("t1").set("plotgroup", "pg2");
          model.result("pg2").run();
          
          model.result().create("pg3", "PlotGroup1D");
          model.result("pg3").label("Temperature based");
          
          model.result("pg3").create("glob1", "Global");
          model.result("pg3").feature("glob1").label("Stress/T");
          model.result("pg3").feature("glob1").set("data", "dset1");
          
          model.result("pg3").feature("glob1").setIndex("expr", "aveop1(solid.sx)", 0);
          model.result("pg3").feature("glob1").setIndex("expr", "Ms(aveop1(T))", 1);
          model.result("pg3").feature("glob1").setIndex("expr", "Mf(aveop1(T))", 2);
          model.result("pg3").feature("glob1").setIndex("expr", "As(aveop1(T))", 3);
          model.result("pg3").feature("glob1").setIndex("expr", "Af(aveop1(T))", 4);
          
          model.result("pg3").feature("glob1").set("xdata", "expr");
          model.result("pg3").feature("glob1").set("xdataexpr", "aveop1(T)");
          model.result("pg3").set("axislimits", true);
          model.result("pg3").set("xmin", "TMf_1-50");
          model.result("pg3").set("xmax", "TAf_1+50");
          
          model.result("pg3").run();



          return model;
      }

      public static void main(String[] args) {
          Model model = run();
      }
}
