/*
 * Draft.java
 */

import com.comsol.model.*;
import com.comsol.model.util.*;

/** Model exported on Aug 18 2020, 14:15 by COMSOL 5.5.0.359. */
public class Draft {

  public static Model run() {
    Model model = ModelUtil.create("Model");

    model.modelPath("/home/apadhy/projects/PhD/COMSOL");
    model.label("Wendler.mph");

    model.param().label("External Parameters");
    
    model.param().set("thickness", "0.03[mm]");
    model.param().descr("thickness", "sample thickness");
    model.param().set("width", "1[mm]");
    model.param().descr("width", "sample width");
    model.param().set("length", "10.0[mm]");
    model.param().descr("length", "clamping length");
    model.param().set("Tambient", "60[degC]");
    model.param().descr("Tambient", "ambient temperature");
    model.param().set("ht", "30[W/(m^2*K)]");
    model.param().descr("ht", "heat transfer coefficient");
    model.param().set("t0", "0[s]");
    model.param().descr("t0", "start time");
    model.param().set("tf", "2[s]");
    model.param().descr("tf", "end time");
    model.param().set("load", "250e6[Pa]");
    model.param().set("t_ramp", "0.25[s]");
    model.param().set("t_rise", "0.2[s]");
    model.param().set("t_decay", "0.2[s]");
    model.param().set("t_eff", "0.3[s]");
    model.param().set("tstart", "0.1[s]");
    model.param().descr("tstart", "waiting time at beginning");
    model.param().set("teq", "0.001[s]");
    model.param().descr("teq", "time for temperature equilibriation");
    model.param().set("strainrate", "0.5[1/s]");
    model.param().set("strainmax", "0.45[mm]");
    model.param().descr("strainmax", "maximum strain (load point)");
    model.param().set("t1", "strainmax/(length*strainrate)");
    model.param().set("speed", "strainrate*length");
    model.param().descr("speed", "absolute strainrate");
    
    
    model.param().group().create("par2");
    model.param("par2").label("Solver parameters1");
    model.param("par2").set("dt", "0.001[s]");
    model.param("par2").descr("dt", "time step");


    model.func().create("pw1", "Piecewise");
    model.func("pw1").label("Displacement boundary 1");
    model.func("pw1").set("funcname", "displ2");
    model.func("pw1").set("arg", "t");
    model.func("pw1").set("smooth", "contd2");
    model.func("pw1").set("smoothzone", "0.01");
    model.func("pw1").set("smoothends", true);
    
    model.func("pw1").setIndex("pieces", 0, 0, 0);
    model.func("pw1").setIndex("pieces", "tstart", 0, 1);
    model.func("pw1").setIndex("pieces", "1e-8", 0, 2);
    
    model.func("pw1").setIndex("pieces", "tstart", 1, 0);
    model.func("pw1").setIndex("pieces", "tstart+t1", 1, 1);
    model.func("pw1").setIndex("pieces", "speed*(t-tstart)", 1, 2);
    
    model.func("pw1").setIndex("pieces", "tstart+t1", 2, 0);
    model.func("pw1").setIndex("pieces", "tstart+t1+teq", 2, 1);
    model.func("pw1").setIndex("pieces", "strainmax", 2, 2);
    
    model.func("pw1").setIndex("pieces", "tstart+t1+teq", 3, 0);
    model.func("pw1").setIndex("pieces", "tstart+2*t1+teq", 3, 1);
    model.func("pw1").setIndex("pieces", "strainmax-speed*(t-(tstart+t1+teq))", 3, 2);
    model.func().create("pw2", "Piecewise");
    
    
    model.func("pw2").label("Displacement boundary");
    model.func("pw2").set("funcname", "displ1");
    model.func("pw2").set("arg", "t");
    model.func("pw2").set("smooth", "contd2");
    model.func("pw2").set("smoothzone", "0.01");
    model.func("pw2").set("smoothends", true);
    
    
    model.func("pw2").setIndex("pieces", 0, 0, 0);
    model.func("pw2").setIndex("pieces", "tstart", 0, 1);
    model.func("pw2").setIndex("pieces", "1e-8", 0, 2);
    
    model.func("pw2").setIndex("pieces", "tstart", 1, 0);
    model.func("pw2").setIndex("pieces", "tstart+strainmax/speed", 1, 1);
    model.func("pw2").setIndex("pieces", "speed*(t-tstart)", 1, 2);
    
    model.func("pw2").setIndex("pieces", "tstart+strainmax/speed", 2, 0);
    model.func("pw2").setIndex("pieces", "tstart+strainmax/speed+teq", 2, 1);
    model.func("pw2").setIndex("pieces", "strainmax", 2, 2);
    
    model.func("pw2").setIndex("pieces", "tstart+strainmax/speed+teq", 3, 0);
    model.func("pw2").setIndex("pieces", "tstart+2*strainmax/speed+teq", 3, 1);
    model.func("pw2").setIndex("pieces", "strainmax-speed*(t - (tstart+strainmax/speed+teq))", 3, 2);
    
    
    model.func().create("pw3", "Piecewise");
    model.func("pw3").label("load boundary");
    model.func("pw3").set("arg", "t");
    model.func("pw3").set("smooth", "contd2");
    model.func("pw3").set("smoothzone", "0.001");
    model.func("pw3").set("smoothends", true);
    model.func("pw3").set("funcname", "extforce");
    
    model.func("pw3").setIndex("pieces", 0, 0, 0);
    model.func("pw3").setIndex("pieces", "teq", 0, 1);
    model.func("pw3").setIndex("pieces", 0, 0, 2);
    
    model.func("pw3").setIndex("pieces", "teq", 1, 0);
    model.func("pw3").setIndex("pieces", "teq+t_rise", 1, 1);
    model.func("pw3").setIndex("pieces", "(load/t_rise)*(t-teq)", 1, 2);
    
    model.func("pw3").setIndex("pieces", "teq+t_rise", 2, 0);
    model.func("pw3").setIndex("pieces", "teq+t_rise+t_eff", 2, 1);
    model.func("pw3").setIndex("pieces", "load", 2, 2);
    
    model.func("pw3").setIndex("pieces", "teq+t_rise+t_eff", 3, 0);
    model.func("pw3").setIndex("pieces", "teq+t_rise+t_eff+t_decay", 3, 1);
    model.func("pw3").setIndex("pieces", "load-(load/t_decay)*(t-teq-t_rise-t_eff)", 3, 2);
    
    model.func("pw3").setIndex("pieces", "teq+t_rise+t_eff+t_decay", 4, 0);
    model.func("pw3").setIndex("pieces", "tf", 4, 1);
    model.func("pw3").setIndex("pieces", 0, 4, 2);
    
    
    model.func().create("pw4", "Piecewise");
    model.func("pw4").label("ambient temperature ramp ");
    model.func("pw4").set("funcname", "extemp");
    model.func("pw4").set("arg", "t");
    model.func("pw4").set("smooth", "contd2");
    model.func("pw4").set("smoothzone", "0.01");
    model.func("pw4").set("smoothends", true);
    
    model.func("pw4").setIndex("pieces", 0, 0, 0);
    model.func("pw4").setIndex("pieces", "2*tstart+2*strainmax/speed+teq", 0, 1);
    model.func("pw4").setIndex("pieces", "Tambient", 0, 2);
    
    model.func("pw4").setIndex("pieces", "2*tstart+2*strainmax/speed+teq", 1, 0);
    model.func("pw4").setIndex("pieces", "2*tstart+3*strainmax/speed+teq", 1, 1);
    model.func("pw4")
         .setIndex("pieces", "Tambient+(50[K]/(strainmax/speed))*(t-(teq+2*tstart+2*strainmax/speed))", 1, 2);
    
    model.func("pw4").setIndex("pieces", "2*tstart+3*strainmax/speed+teq", 2, 0);
    model.func("pw4").setIndex("pieces", "2*tstart+3*strainmax/speed+2*teq", 2, 1);
    model.func("pw4").setIndex("pieces", "Tambient+50[K]", 2, 2);
    
    model.func("pw4").setIndex("pieces", "2*tstart+3*strainmax/speed+2*teq", 3, 0);
    model.func("pw4").setIndex("pieces", "2*tstart+4*strainmax/speed+2*teq", 3, 1);
    model.func("pw4")
         .setIndex("pieces", "Tambient+50[K]-(50[K]/(strainmax/speed))*(t-(2*teq+2*tstart+3*strainmax/speed))", 3, 2);
    
    
    model.func().create("pw5", "Piecewise");
    model.func("pw5").label("ambient temperature ramp 1");
    model.func("pw5").set("funcname", "extemp_force");
    model.func("pw5").set("arg", "t");
    model.func("pw5").set("smooth", "contd2");
    model.func("pw5").set("smoothzone", "0.001");
    model.func("pw5").set("smoothends", true);
    
    model.func("pw5").setIndex("pieces", 0, 0, 0);
    model.func("pw5").setIndex("pieces", "2*teq+t_rise+t_eff+t_decay", 0, 1);
    model.func("pw5").setIndex("pieces", "Tambient", 0, 2);
    
    model.func("pw5").setIndex("pieces", "2*teq+t_rise+t_eff+t_decay", 1, 0);
    model.func("pw5").setIndex("pieces", "2*teq+2*t_rise+t_eff+t_decay", 1, 1);
    model.func("pw5").setIndex("pieces", "Tambient+(350[K]/t_rise)*(t-(2*teq+t_rise+t_eff+t_decay))", 1, 2);
    
    model.func("pw5").setIndex("pieces", "2*teq+2*t_rise+t_eff+t_decay", 2, 0);
    model.func("pw5").setIndex("pieces", "2*teq+2*t_rise+2*t_eff+t_decay", 2, 1);
    model.func("pw5").setIndex("pieces", "Tambient+350[K]", 2, 2);
    
    model.func("pw5").setIndex("pieces", "2*teq+2*t_rise+2*t_eff+t_decay", 3, 0);
    model.func("pw5").setIndex("pieces", "2*teq+2*t_rise+2*t_eff+2*t_decay", 3, 1);
    model.func("pw5")
         .setIndex("pieces", "Tambient+350[K]-(450[K]/t_decay)*(t-(2*teq+2*t_rise+2*t_eff+t_decay))", 3, 2);
    
    model.func("pw5").setIndex("pieces", "2*teq+2*t_rise+2*t_eff+2*t_decay", 4, 0);
    model.func("pw5").setIndex("pieces", "tf", 4, 1);
    model.func("pw5").setIndex("pieces", "Tambient-100", 4, 2);

    
        
    model.component().create("comp1", true);
    model.component("comp1").geom().create("geom1", 2);
    model.component("comp1").variable().create("var1");
    model.component("comp1").variable("var1").label("SMA model");
    
    model.component("comp1").variable("var1").set("EA", "38.4e9[Pa]");
    model.component("comp1").variable("var1").descr("EA", "A Young's modulus");
    
    model.component("comp1").variable("var1").set("EM", "29.0e9[Pa]");
    model.component("comp1").variable("var1").descr("EM", "M Young's modulus");
    
    model.component("comp1").variable("var1").set("s0_AM", "400.0[MPa]");
    model.component("comp1").variable("var1").descr("s0_AM", "A-M Stress barrier at Tref");
    
    model.component("comp1").variable("var1").set("s0_MA", "150.0[MPa]");
    model.component("comp1").variable("var1").descr("s0_MA", "M-A Stress barrier at Tre");
    
    model.component("comp1").variable("var1").set("epsT", "0.042");
    model.component("comp1").variable("var1").descr("epsT", "transformation strain");
    
    model.component("comp1").variable("var1").set("cAM", "4.8[MPa/K]");
    model.component("comp1").variable("var1").descr("cAM", "A-M Clausius Coefficient");
    
    model.component("comp1").variable("var1").set("cMA", "5.88[MPa/K]");
    model.component("comp1").variable("var1").descr("cMA", "M-A Clausius-Coefficient");
    
    model.component("comp1").variable("var1").set("young2a", "(xA/EA+xi/EM)^-1");
    model.component("comp1").variable("var1").descr("young2a", "Modulus (mixture)");
    
    model.component("comp1").variable("var1").set("k_het", "(xA/kappaA+xi/kappaM)^-1");
    model.component("comp1").variable("var1").descr("k_het", "thermal conductivity (mixture)");
    
    model.component("comp1").variable("var1").set("Tref", "346[K]");
    model.component("comp1").variable("var1").descr("Tref", "Reference temperature for stress barriers");
    
    model.component("comp1").variable("var1").set("TM", "-5[degC]");
    model.component("comp1").variable("var1").descr("TM", "austenite stability limit");
    
    model.component("comp1").variable("var1").set("L", "22.13[J/g]");
    model.component("comp1").variable("var1").descr("L", "latent heat");
    
    model.component("comp1").variable("var1").set("kappaA", "0.18[W/cm/K]");
    model.component("comp1").variable("var1").descr("kappaA", "A thermal conductivity");
    
    model.component("comp1").variable("var1").set("kappaM", "0.086[W/cm/K]");
    model.component("comp1").variable("var1").descr("kappaM", "M thermal conductivity");
    
    model.component("comp1").variable("var1")
         .set("hp", "0.5*(1/EM-1/EA)/1.04396*stress^2+stress*epsT+0.18e6[J/m^3/K]*(T-Tref)+7.5e7[J/m^3]");
    model.component("comp1").variable("var1").descr("hp", "transformation enthalpy A-M+");
    
    model.component("comp1").variable("var1")
         .set("hm", "0.5*(1/EM-1/EA)/1.04396*stress^2-stress*epsT+0.18e6[J/m^3/K]*(T-Tref)+7.5e7[J/m^3]");
    model.component("comp1").variable("var1").descr("hm", "transformation enthalpy A-M-");
    
    model.component("comp1").variable("var1").set("stress", "sign(solid.I1s)*solid.tresca");
    model.component("comp1").variable("var1").descr("stress", "scalar stress");
    
    
    model.component("comp1").variable().create("var2");
    model.component("comp1").variable("var2").label("Kinetic Model");
    
    model.component("comp1").variable("var2").set("xi", "xMm + xMp");
    model.component("comp1").variable("var2").descr("xi", "total martensite fraction");
    
    model.component("comp1").variable("var2").set("xA", "1 - xMm -xMp");
    model.component("comp1").variable("var2").descr("xA", "austenite fraction");
    
    model.component("comp1").variable("var2").set("tresca1", "abs(solid.sy-solid.sx)");
    model.component("comp1").variable("var2").descr("tresca1", "Tresca stress");
    
    model.component("comp1").variable("var2").set("stress1", "sign(solid.I1s)*solid.tresca");
    model.component("comp1").variable("var2").descr("stress1", "scalar stress");
    
    model.component("comp1").variable("var2").set("V_D", "5e-23[m^3]");
    model.component("comp1").variable("var2").descr("V_D", "MAS microscopic layer volume");
    
    model.component("comp1").variable("var2").set("kB", "1.380649e-23[J/K]");
    model.component("comp1").variable("var2").descr("kB", "Boltzmann constant");
    
    model.component("comp1").variable("var2").set("escaleMA", "V_D/(2*1.04396*EM*kB)");
    model.component("comp1").variable("var2").set("escaleAM", "V_D/(2*1.04396*EA*kB)");
    
    model.component("comp1").variable("var2").set("sqrEMA", "sqrt(EM/EA)");
    
    model.component("comp1").variable("var2").set("sigma_A", "(s0_AM+cAM*(T-Tref))*(1+noise(x,y))");
    model.component("comp1").variable("var2").descr("sigma_A", "austenite plateau stress");
    
    model.component("comp1").variable("var2").set("p_AMp", "if(T<=TM, 1/tau, p_AMp_2)");
    model.component("comp1").variable("var2").descr("p_AMp", "trans. prob. A to M+");
    
    
    model.component("comp1").variable("var2")
         .set("p_AMp_2", "if( stress<sigma_A, 1/tau*(1-erf(0.91*sqrt(escaleAM/T)*(sigma_A-stress))),  1/(tau*erf(2*sqrt(escaleAM/T)*sigma_A) ) )");
    
    model.component("comp1").variable("var2").set("p_AMm", "if(T<=TM, 1/tau, p_AMm_2)");
    
    model.component("comp1").variable("var2").descr("p_AMm", "trans. prob. A to M-");
    
    model.component("comp1").variable("var2")
         .set("p_AMm_2", "if( -stress<sigma_A, 1/tau*(1-erf(0.91*sqrt(escaleAM/T)*(sigma_A+stress))),  1/(tau*erf(2*sqrt(escaleAM/T)*sigma_A) ) )");
    
    model.component("comp1").variable("var2").set("sigma_M", "(s0_MA+cMA*(T-Tref))*(1+noise(x,y))");
    model.component("comp1").variable("var2").descr("sigma_M", "martensite plateau stress");
    
    model.component("comp1").variable("var2").set("p_MpA", "if( stress < sigma_M, sqrEMA/tau, p_MpA_2)");
    model.component("comp1").variable("var2").descr("p_MpA", "trans. prob. M+ to A");
    
    model.component("comp1").variable("var2")
         .set("p_MpA_2", "sqrEMA/tau*(1-erf(-0.91*sqrt(escaleMA/T)*(sigma_M-stress)))");
    
    model.component("comp1").variable("var2").set("p_MmA", "if( -stress<sigma_M, sqrEMA/tau, p_MmA_2)");
    model.component("comp1").variable("var2").descr("p_MmA", "trans. prob. M- to A");
    
    model.component("comp1").variable("var2")
         .set("p_MmA_2", "sqrEMA/tau*(1-erf(-0.91*sqrt(escaleMA/T)*(sigma_M+stress)))");
    
    model.component("comp1").variable("var2").set("gamma", "4e-8");
    model.component("comp1").variable("var2").descr("gamma", "PFM Interfacial free energy parameter");
    
    model.component("comp1").variable("var2").set("delta", "2e-4");
    model.component("comp1").variable("var2").descr("delta", "PFM interface width");
    
    model.component("comp1").variable("var2").set("omega", "1e-3");
    model.component("comp1").variable("var2").descr("omega", "PFM kinetic coefficient");
    
    model.component("comp1").variable("var2").set("tau", "1e-3 [s]");
    model.component("comp1").variable("var2").descr("tau", "MAS relaxation time");
    
    
    model.component("comp1").variable().create("var3");
    model.component("comp1").variable("var3").label("Deviatoric stress tensor");
    model.component("comp1").variable("var3").set("dev111", "solid.sdevx/solid.mises");
    model.component("comp1").variable("var3").set("dev22", "solid.sdevy/solid.mises");
    model.component("comp1").variable("var3").set("dev33", "solid.sdevz/solid.mises");
    model.component("comp1").variable("var3").set("dev12", "solid.sdevxy/solid.mises");
    model.component("comp1").variable("var3").set("dev13", "solid.sdevxz/solid.mises");
    model.component("comp1").variable("var3").set("dev23", "solid.sdevyz/solid.mises");
    model.component("comp1").variable("var3").active(false);

    model.component("comp1").func().create("rn1", "Random");
    model.component("comp1").func("rn1").set("nargs", 2);
    model.component("comp1").func("rn1").set("uniformrange", 0.001);
    model.component("comp1").func("rn1").set("funcname", "noise");
    
    model.component("comp1").common().create("ampr1", "AmbientProperties");
    model.component("comp1").common("ampr1").set("name", "amth_ht");
    model.component("comp1").common("ampr1").label("Ambient Thermal Properties (ht)");

    model.component("comp1").geom("geom1").create("r1", "Rectangle");
    model.component("comp1").geom("geom1").feature("r1").set("size", new String[]{"length", "width"});
    model.component("comp1").geom("geom1").run("r1");
    model.component("comp1").geom("geom1").feature("fin").set("repairtoltype", "relative");
    model.component("comp1").geom("geom1").run("fin");
    

    return model;
  }

  public static void main(String[] args) {
    run();
  }

}
